@@ -13,10 +13,11 @@ from dbstream.tools.parse_data import treat_json_data
 from dbstream.tools.regex import replace_query_details
 from dbstream.tunnel import create_ssh_tunnel
 
+from google.api_core.exceptions import Forbidden
 
 class DBStream:
 
-    def __init__(self, instance_name, client_id, special_env=None):
+    def __init__(self, instance_name, client_id, special_env=None, id_info='dck'):
         self.instance_name = instance_name
         self.instance_type_prefix = ""
         self.ssh_init_port = ""
@@ -25,6 +26,7 @@ class DBStream:
         unix_time = time.mktime(datetime.datetime.now().timetuple())
         self.dbstream_instance_id = 'df-' + str(unix_time) + '-' + str(random.randint(1000, 9999))
         self.special_env = special_env if special_env else None
+        self.id_info = id_info
 
     def error_if_function_not_exist(self, function_name):
         raise Exception("Function %s is not defined for %s" % (function_name, type(self).__name__))
@@ -140,9 +142,9 @@ class DBStream:
                 r = requests.post(url=url, data=json.dumps(body))
                 print(r.status_code)
 
-    def send(self, data, replace=False, apply_special_env=True, **kwargs):
+    def send(self, data, replace=False, apply_special_env=True, delay=5, batch_id=None, **kwargs):
         # data['data'] = generate_dck_info(data['data'])
-        list_of_tables_to_send, list_of_pop_fields = treat_json_data(data)
+        list_of_tables_to_send, list_of_pop_fields = treat_json_data(data, batch_id=batch_id, id_info=self.id_info)
         for d in list_of_tables_to_send:
             # if d.get('table_name') == 'test.test_orders_fulfillments_line_items':
             #     print(d.get('data'))
@@ -161,7 +163,11 @@ class DBStream:
                 "table_name": d.get('table_name')
             }
             if len(data_to_send['columns_name']) > 0:
-                self.send_data(data=data_to_send, replace=replace, apply_special_env=apply_special_env, **kwargs)
+                try:
+                    self.send_data(data=data_to_send, replace=replace, apply_special_env=apply_special_env, **kwargs)
+                except Forbidden:
+                    time.sleep(delay)
+                    self.send_data(data=data_to_send, replace=replace, apply_special_env=apply_special_env, **kwargs)
 
     def send_temp_data(self, data, schema_prefix, table, column_names, apply_special_env=True):
         data_to_send = {
